K = require('kcore')

class K.Auth
  constructor: (@kRequest) ->
    K.Error.throwExceptionIfParameterNotSet(@kRequest, 'kRequest')

    @app    = @kRequest.getApp()
    @appLog = @kRequest.getAppLog()

    K.Error.throwExceptionIfParameterNotSet(@app, 'app')
    K.Error.throwExceptionIfParameterNotSet(@appLog, 'appLog')

  getCurrentUserAuthClass: (done) ->
    @kRequest.getSessionForRead (err, session) =>
      if err?
        # Error while getting session object.
        done(err)

      else
        # We got session. Get auth class from it.
        authClass = session.getOrDefault('auth.class', 'default')
        session.release()
        done(null, authClass)

  getCurrentUserModel: (done) ->
    @kRequest.getSessionForRead (err, session) =>
      if err?
        # Error while getting session object.
        done(err)

      else
        # We got session. Get auth class from it.
        authClass = session.getOrDefault('auth.class', 'default')
        authId    = session.getOrDefault('auth.id', 'default')
        session.release()

        # Pass control to auth privder.
        authProvider = @kRequest.getAuthProvider(authClass)
        authProvider.getAuthenticatedUserModelByAuthId(authId, done)

  selectIdentityAsCurrent: (authClass, authId, done) ->
    authProvider = @kRequest.getAuthProvider(authClass)
    authProvider.isAuthenticatedIdentity authId, (err, isAuth) =>
      if err?
        # Error while checking is user logged in.
        @appLog.logIfError(err)
        done(err)

      else if isAuth
        # User logged in. Set new auth class.
        @kRequest.getSessionForWrite (err, session) =>
          if err?
            # Error while getting session object.
            done(err)

          else
            # We got session. Set new auth data.
            session.setData('auth.class', authClass)
            session.setData('auth.id', authId)
            session.release(done)

      else
        # User not logged in.
        err = new K.Error({errorCode:'noAuth', ext:{authClass:authClass, authId:authId}})
        @appLog.logIfError(err)
        done(err)

  getIdentitiesOfClass: (authClass, done) ->
    authProvider = @kRequest.getAuthProvider(authClass)
    authProvider.getAuthenticatedIdentities(done)

  cleanUp: (done) ->
    # User logged in. Set new auth class.
    @kRequest.getSessionForWrite (err, session) =>
      if err?
        # Error while getting session object.
        done(err)

      else
        # We got session. Set new auth data.
        session.resetData('auth.class')
        session.resetData('auth.id')
        session.release(done)
