let K = require('kcore')
let process = require('process')
let async = require('async')
let deasync = require('deasync')
let fs = require('fs')

class CascadingConfigLoader
{
  constructor(defaultConfig = {})
  {
    this.currentConfig = defaultConfig
  }

  getConfig()
  {
    return this.currentConfig
  }

  load(filename, ctx, done)
  {
    if ((typeof ctx == 'undefined') || (ctx == null))
    {
      ctx = {}
    }

    filename = K.FileUtil.normalizePath(filename)

    let callingPathExploded = []
    let callingPathIter     = ''

    if (filename.indexOf('/') == -1)
    {
      // Filename given only (e.g. config.js) - use working directory.
      let callingPath = K.FileUtil.normalizePath(process.cwd())
      callingPathExploded = callingPath.split('/')
    }
    else
    {
      // Full path to config file given - use it.
      callingPathExploded = filename.split('/')
      filename            = callingPathExploded[callingPathExploded.length - 1]
      callingPathExploded = callingPathExploded.slice(0, -1)
    }

    // Process config files one by one beginning from root directory:
    // /config.js
    // /a/config.js
    // /a/b/config.js
    // /a/b/.../config.js
    async.eachSeries(callingPathExploded,
      (token, cb) =>
      {
        // Process next path token (go one level deeper)
        callingPathIter += token + '/'
        ctx.currentFilepath = callingPathIter + filename

        fs.exists(
          ctx.currentFilepath,
          (exists) =>
          {
            if (exists)
            {
              // Config file exists, process it.
              require(ctx.currentFilepath).makeConfig(this.currentConfig, ctx, cb)
            }
            else
            {
              // Config file does not exist, skip this one and go on.
              cb(null)
            }
          }
        )
      },
      function(err)
      {
        // Done - all config files processed.
        done(null)
      }
    )
  }

  loadSync(filename, ctx)
  {
    return deasync(this.load).bind(this)(filename, ctx)
  }
}

//
// Export classes.
//

K.CascadingConfigLoader = CascadingConfigLoader
