let K  = require('kcore')
let fs = require('fs')

class FileTimestampTracker
{
  constructor(path)
  {
    this.path      = path
    this.timestamp = null
  }

  _getCurrentTimestampFromFileSystem(done)
  {
    fs.stat(this.path, (err, fileinfo) =>
    {
      if (err)
      {
        // Error while stating file.
        done(err)
      }
      else
      {
        // File info ready, passt last modify time to caller.
        done(null, fileinfo.mtime)
      }
    })
  }

  getPath()
  {
    return this.path
  }

  getTimestamp()
  {
    return this.timestamp
  }

  reloadIfChanged(reloadCb, readyCb)
  {
    this._getCurrentTimestampFromFileSystem((err, currentTimestamp) =>
    {
      if (err)
      {
        // Error while asking filesystem for current timestamp.
        readyCb(err)
      }
      else
      {
        // Current timestamp retrived. Check is it the same as at the last call.
        if ((this.timestamp == null) || (currentTimestamp.getTime() != this.timestamp.getTime()))
        {
          // Timestamp changed. Send reaload signal before ready.
          reloadCb((err) =>
          {
            this.timestamp = currentTimestamp

            readyCb(err)
          })
        }
        else
        {
          // Nothing changed, just call ready callback.
          readyCb(null)
        }
      }
    })
  }
}

// Export class to K namespace.
K.FileTimestampTracker = FileTimestampTracker
