K     = require('kcore')
Async = require('async')

class K.LockManager
  constructor: (app) ->
    # Check args
    K.Error.throwExceptionIfParameterNotSet(app, 'app')
    K.Error.throwExceptionIfParameterNotSet(app.getLog(), 'app.getLog()')

    # Init object
    @app          = app
    @appLog       = app.getLog()
    @_accessQueue = {}

  _exclusiveAccessReady: (lockHandle, done) =>
    @appLog.logTime {eventType: 'exclusiveAccessReady', resourceName: lockHandle.resourceName}
    # Pass control to waiter.
    lockHandle.close = done
    lockHandle.doWorkCb(null, lockHandle)

  openExclusiveAccess: (resourceName, done) ->
    if not @_accessQueue[resourceName]?
      # Create new queue if does not exist yet.
      queue = Async.queue(@_exclusiveAccessReady.bind(@), 1)
      @_accessQueue[resourceName] = queue

      # Remove queue if last waiter popped
      queue.drain = () =>
        delete @_accessQueue[resourceName]

    # Push task to queue.
    lockHandle =
      close: null,
      doWorkCb: done,
      resourceName: resourceName

    @_accessQueue[resourceName].push(lockHandle)
    @appLog.logTime {eventType: 'openExclusiveAccess', resourceName: resourceName}
