K = require('kcore')
express = require('express')
vhost = require('vhost')

class K.ServerVHost
  constructor: (@config) ->
    @vhosts = {}

    # log
    @vHostLog = null
    if @config.serverLog
      @serverLog = @config.serverLog
      @serverLog.log('Starting VHost server')
      @vHostLog = new K.LogCtxApp(@serverLog, @)
    else
      # no log, so attach empty call
      @vHostLog = {log: () -> ;}

    @serverStats = new K.ServerStatsCollector()

    # routing app
    @vhostsRoutingApp = express()
    @vhostsRoutingApp.use (req, res, next) =>
      req.log = new K.LogCtxRequest(@vHostLog, req)
      req.log.logRequest()
      next()

  getLog: () ->
    return @vHostLog

  getName: () ->
    return 'vHost.root'

  addVHost: (hostname, kserverapp) ->
    @vhostsRoutingApp.use(vhost(hostname, kserverapp.getExpress()))
    @vhosts[hostname] = kserverapp

  onGotWsConnection: (ws) ->
    @serverStats.reportWebsocketOpen()
    connectionHost = ws.upgradeReq.headers.host
    for vhostName of @vhosts
      if connectionHost.indexOf(vhostName) == 0
        @vhosts[vhostName].onGotWsConnection(ws)
        break

  onLostWsConnection: (ws) ->
    @serverStats.reportWebsocketClose()
    ws.app?.onLostWsConnection(ws)

  _onUnknownHost: (req, res, next) ->
    @nateRoot = new K.NateHtmlDocument()
    @nateRoot.body().newP('Unknown host [' + req.hostname + ']!')

    res.status(404).send(@nateRoot.render())
    @vHostLog.logTime {eventType: 'unknownHost', host: req.hostname}
    next()

  start: () ->
    # Handle unknown hosts.
    @vhostsRoutingApp.use(vhost(/.*/, @_onUnknownHost.bind(@)))

    httpServer = @vhostsRoutingApp.listen @config.port, () =>
      host = httpServer.address().address
      port = httpServer.address().port
      console.log 'V-HOSTS server listening at http://%s:%s', host, port

    # WebSockets Server
    if not @config.testJsonActionsDead and @config.useWebSocketServer
      @wsServer = new K.WebSocketServer(@, {server: httpServer})

    # request statistics
    K.ServerStatsCollector.setRequestStatsHandler(httpServer, @serverStats)
