K = require('kcore')

WS_WebSocket = require("ws")
WS_Server = require("ws").Server

_LOG_WEBSOCKETS_MANAGE_ = 0

class K.WebSocketServer
  constructor: (@master, @config) ->
    @uniqueConnectionId = 0
    @connections  = {}

    # WebSockets
    @wss = new WS_Server(@config)
    @wss.on 'connection', (ws, req) =>
      ws.upgradeReq = req
      @_gotConnection(ws)

  _gotConnection: (ws) =>
    # add connection to list
    @uniqueConnectionId++
    @connections[@uniqueConnectionId] = ws
    ws.id = @uniqueConnectionId

    # Tell master we got the connection.
    # The master should attach app field, so:
    #   if the master is just the app, it should attach "this"
    #   if the master is non-app (for example vHost server), then it should find
    #   correct app, and attach it.
    @master.onGotWsConnection(ws)

    if not ws.app?
      return
      # TODO: we should do something more clever here, like closing the connection, logging etc.

    # Init log context for new request.
    ws.app._createLogContextForNewRequest(ws.app, ws.upgradeReq)
    ws.upgradeReq.log.logTime {
      eventType: 'Websocket:gotConnection'
      host:      ws.upgradeReq.headers.host
      wsId:      ws.id
    }

    # attach kRequest object to http upgrade header related
    # with websocket.
    ws.app.createKRequest ws.upgradeReq, null, (err, kRequest) =>
      ws.app.appLog.logIfError(err)

      # Inject websocket object into kRequest.
      # After that kRequest should upgrade connection method from ajax to ws.
      if kRequest?
        kRequest.attachWebsocket(ws)

      # setup
      ws.kRequest = kRequest

      ws.log = (msg...) ->
        if _LOG_WEBSOCKETS_MANAGE_
          console.log 'Websocket[id:' + @id + ']', msg
      ws.trySend = (msg) ->
        ws.send msg, (err) =>
          @log 'ws.send() error:', err

      ws.log('connected')
      ws.send 'halo'

      thiz = @

      # setup handlers
      ws.on 'close', (msg) ->
        if _LOG_WEBSOCKETS_MANAGE_
          console.log 'deleting Websocket[id:' + @id + '] with code: ', msg
        delete thiz.connections[@id]

        thiz.master.onLostWsConnection(ws, msg)

        # Tell event server, that connection (krequest) related with given
        # websocket is dead, so it can unregister all related stuff.
        ws.app.getEventServer().onKRequestClosed(@kRequest)

        # Log.
        ws.upgradeReq.log.logTime {
          eventType: 'Websocket:closedConnection'
          wsId:      @id
          msg:       msg
        }
        @log('closed')

      ws.on 'message', (msg) ->
        @log 'rx:', msg
        ws.app.processWsMsg(ws, msg)

      ws.on 'error', (err) ->
        ws.app.getLog().logTime {
          eventType: 'Websocket:error'
          wsId:      @id
          err:       err
        }
        @log 'ws error:', err

  toAll: (data) ->
    for theWssId of @connections
      @connections[theWssId].send(data)